@@ -9,30 +9,55 @@
 
 
 model.diagnostics<-function(	model.obj=NULL,
-				qdata.trainfn=NULL,
-				qdata.testfn=NULL,
-				folder=NULL,		# No ending slash, to output to working dir = getwd()
-				MODELfn=NULL,
-				response.name=NULL,
-				unique.rowname=NULL,	# Row identifier
-				diagnostic.flag=NULL,
-				seed=NULL,
-			# Model Evaluation Arguments
-				prediction.type=NULL,
-				MODELpredfn=NULL,
-				na.action=NULL,	# also used for mapping
-				v.fold=10,
-				device.type=NULL,	# options: "default", "jpeg", "none","postscript"
-				DIAGNOSTICfn=NULL,
-				jpeg.res=72,
-				device.width=7,
-				device.height=7,
-				cex=par()$cex,
-				req.sens,req.spec,FPC,FNC,
-			# SGB arguments
-				n.trees=NULL
+					qdata.trainfn=NULL,
+					qdata.testfn=NULL,
+					folder=NULL,		# No ending slash, to output to working dir = getwd()
+					MODELfn=NULL,
+					response.name=NULL,
+					unique.rowname=NULL,	# Row identifier
+					diagnostic.flag=NULL,
+					seed=NULL,
+				# Model Evaluation Arguments
+					prediction.type=NULL,
+					MODELpredfn=NULL,
+					na.action=NULL, #switch(model.type,CF="na.omit",NULL) ,	# also used for mapping
+					v.fold=10,
+					device.type=NULL,	# options: "default", "jpeg", "none", "png, "postscript", "tiff"
+					DIAGNOSTICfn=NULL,
+				# Graphics Arguments
+					res=NULL,
+					jpeg.res=72,
+					device.width=7,
+					device.height=7,
+					units="in",
+					pointsize=12,
+					cex=par()$cex,
+				# PresenceAbsence arguments
+					req.sens,req.spec,FPC,FNC,
+				# QRF arguments
+					quantiles=NULL,
+					all=TRUE,
+				# CF arguments:
+					subset=NULL, 
+					weights=NULL,
+					#ntree=NULL,
+					mtry=NULL,
+					controls=NULL,
+					xtrafo=NULL, 
+					ytrafo=NULL,
+					scores=NULL,
+				# SGB arguments
+					n.trees=NULL
 ){
 
+#WARN<-NULL
+#WARN<-c(WARN,"first warn")
+#WARN<-c(WARN,"second warn")
+
+#warning("test")
+#warning("test 2")
+
+
 ## Note: Must have R version 2.8 or greater
 ## Note: Must have all rasters in same size, same resolution, and with -9999 for nodata values.
 ## Note: The names of the predictors must be the same as the predictor names in the training/test data.
@@ -89,118 +114,68 @@ if(is.null(model.obj)){
 ##################### Extract Model Type from model.obj #############################
 #####################################################################################
 
-model.type.long<-attr(model.obj,"class")
-if(is.null(model.type.long)){model.type.long <- "unknown"}
-
-#model.type<-switch(model.type.long,	"randomForest"="RF",
-#						"gbm"="SGB",
-#						"unknown")
-
-if("randomForest"%in%model.type.long){
-	model.type<-"RF"
-}else{
-	if("gbm"%in%model.type.long){
-		model.type<-"SGB"
-	}else{
-		model.type<-"unknown"
-	}
-}
-
-if(model.type=="unknown"){
-	stop("model.obj is of unknown type")}
+model.type<-check.model.type(model.obj)
 	
-print(paste("model.type =",model.type))
+#print(paste("model.type =",model.type))
 
 #if (model.type == "SGB") {
 #	warning("ModelMap currently uses OOB estimation to determine optimal number of trees in SGB model when calling gbm.perf in the gbm package but OOB generally underestimates the optimal number of iterations although predictive performance is reasonably competitive however using cv.folds>0 when calling gbm usually results in improved predictive performance but is not yet supported in ModelMap")
 #}
 
+#############################################################################################
+########################### Load modelling packages #########################################
+#############################################################################################
+
+if(model.type=="CF"){REQUIRE.party()}
+if(model.type=="QRF"){REQUIRE.quantregForest()}
+if(model.type=="SGB" || model.type=="QSGB"){REQUIRE.gbm()}
+
+
+#####################################################################################
+####################### should warnings be immeadiate ###############################
+#####################################################################################
+if(model.type=="CF" && prediction.type=="CV"){
+	WARN.IM<-TRUE
+}else{
+	WARN.IM<-FALSE
+}
+
+#####################################################################################
+##################### Check Quantiles and Importance ################################
+#####################################################################################
+
+if(model.type=="QRF"){
+	###quantiles###
+	if(!is.null(quantiles)){
+		if(!is.numeric(quantiles)){                stop("'quantiles' must be numbers between 0 and 1")}
+		if(any(quantiles<=0) || any(quantiles>=1)){stop("'quantiles' must be numbers between 0 and 1")}}
+
+	if("QRF"%in%names(model.obj)){imp.quantiles<-model.obj$QRF$quantiles}else{imp.quantiles<-model.obj$quantiles}
+
+	if(is.null(quantiles)){
+		if(!is.null(imp.quantiles)){quantiles<-imp.quantiles}else{quantiles<-c(.1,.5,.9)}}
+}
+
 #############################################################################################
 ########################### Extract predList from model.obj #################################
 #############################################################################################
 
-
-if(model.type == "RF"){predList<-row.names(model.obj$importance)}
-
-if(model.type == "SGB"){predList<-model.obj$var.names}
+predList<-check.predList(model.obj=model.obj,model.type=model.type)
 
 #############################################################################################
 ####################### Extract Factored Predictors from model.obj ##########################
 #############################################################################################
 
-if(model.type=="SGB"){
-	var.factors<-model.obj$var.type!=0
-	if( any(var.factors)){
-		model.levels<-as.list(1:sum(var.factors))
-		names(model.levels)<-model.obj$var.names[var.factors]
-		for(p in 1:sum(var.factors)){
-			model.levels[[p]]<-model.obj$var.levels[var.factors][[p]]
-		}
-		model.obj$levels<-model.levels
-	}
-}
-
-
-if(model.type=="RF"){
-	var.factors<-!sapply(model.obj$forest$xlevels,identical,0)
-	if( any(var.factors)){
-		model.levels<-as.list(1:sum(var.factors))
-		names(model.levels)<-names(var.factors)[var.factors]
-		for(p in 1:sum(var.factors)){
-			model.levels[[p]]<-model.obj$forest$xlevels[var.factors][[p]]
-		}
-		model.obj$levels<-model.levels
-	}
-}
-
-
-#############################################################################################
-######################### Extract response.name from model.obj ##############################
-#############################################################################################
-
-
-if(!is.null(model.obj$response)){
-	model.response.name<-model.obj$response
-	#print(paste("     model.response.name =",model.response.name))
-	#print(paste("     response.name =",response.name))
-	if(is.null(response.name)){
-		response.name<-model.response.name
-	}else{
-		if(response.name!=model.response.name){
-			warning("supplied model.obj has response ",model.response.name," not supplied response ",response.name)
-		}
-	}
-}
-
-
-## If the response variable is NULL, then the user selects variable from pop-up list.
-if (is.null(response.name)){
-	response.name <- select.list(names(qdata), title="Select response name.")
-	if(response.name=="" || is.null(response.name)){
-		stop("response.name is needed")}	
-}
-
-print(paste("response.name =",response.name))
+model.levels<-check.model.levels(model.obj=model.obj,model.type=model.type)
 
+predFactor<-NULL
+if(!is.null(model.levels)){predFactor<-names(model.levels)}
 
 #############################################################################################
 ######################## Extract  response.type from model.obj ##############################
 #############################################################################################
 
-if(model.type=="RF"){
-	response.type<-switch(model.obj$type,"regression"="continuous","classification"="classification","unknown")
-	if(response.type=="classification"){
-		if(identical(levels(model.obj$y),c("0","1"))){
-			response.type<-"binary"
-		}else{
-			response.type<-"categorical"}}}
-if(model.type=="SGB"){
-	response.type<-switch(model.obj$distribution$name,"gaussian"="continuous","bernoulli"="binary","unknown")}
-if(response.type=="unknown"){stop("supplied model.obj has an unknown response type")}
-
-print(paste("response.type=",response.type))
-
-
+response.type<-check.response.type(model.obj=model.obj,model.type=model.type,ONEorTWO="model.obj")
 
 #############################################################################################
 ######################### Select and Check Prediction Type ##################################
@@ -209,6 +184,10 @@ print(paste("response.type=",response.type))
 if (is.null(prediction.type)){
 	if(model.type=="RF"){
 		prediction.type <- select.list(c("TEST","CV","OOB"), title="Select prediction type.")}
+	if(model.type=="QRF"){
+		prediction.type <- select.list(c("TEST","CV","OOB"), title="Select prediction type.")}
+	if(model.type=="CF"){
+		prediction.type <- select.list(c("TEST","CV","OOB"), title="Select prediction type.")}
 	if(model.type=="SGB"){
 		prediction.type <- select.list(c("TEST","CV","TRAIN"), title="Select prediction type.")}
 }
@@ -216,15 +195,48 @@ if (is.null(prediction.type)){
 if(prediction.type=="" || is.null(prediction.type)){
 	stop("prediction.type is needed")}
 if(prediction.type=="TRAIN"){
-	warning("predictions will be made made on the training data and will yeild unrealistic accuracy estimates with regard to independant data")}
+	warning("predictions will be made made on the training data and will yeild unrealistic accuracy estimates with regard to independant data",immediate. = WARN.IM)}
 
 if(model.type=="SGB" && prediction.type=="OOB"){
 	stop("'OOB' predictions not available for 'SGB' models")}
 
-if(model.type=="RF" && prediction.type=="TRAIN"){
+if(model.type%in%c("RF","QRF","CF") && prediction.type=="TRAIN"){
 	stop("'TRAIN' predictions not available for 'RF' models")}
 
 
+#############################################################################################
+############################ extract and check CF arguments #################################
+#############################################################################################
+
+if(prediction.type=="CV" && model.type=="CF"){
+
+	print("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++")
+	warning("For cross validation,", "it is your responsibility to make sure CF arguments match those used to build initial model", immediate. = WARN.IM)
+	print("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++")
+
+	ntree<-length(model.obj@ensemble)
+	#mtry<-get("mtry",model.obj@data@env) #doesn't work
+	
+	if(!is.null(controls)){
+		if(!is.null(mtry)){
+			warning(	"Because 'controls' is specified, individual arguments 'ntree' and 'mtry' ignored,
+	        		 	for values other than default, these parameters must be specified inside of 'controls()'",
+					immediate. = WARN.IM)}
+	}else{
+		if(is.null(mtry)){
+			warning("For CV predictions, neither mtry nor controls specified, using default 'mtry=min(5, Npred-1)'", immediate. = WARN.IM)
+			mtry<-min(5,length(predList)-1)}
+		controls<-party::cforest_unbiased(ntree=ntree,mtry=mtry)
+	}
+	if(is.null(xtrafo)){xtrafo<-party::ptrafo}
+	if(is.null(ytrafo)){ytrafo<-party::ptrafo}
+
+	if(!is.null(subset) || !is.null(weights) || !is.null(scores)){
+		stop("Currently ModelMap does not support cross validation prediction for CF models using 'subset' 'weights' or 'scores'")}
+}
+
+
+
 #############################################################################################
 ################# Extract sampsize and strata from model.obj ################################
 #############################################################################################
@@ -254,29 +266,6 @@ if(is.null(folder)){
 		folder<-getwd()}
 }
 
-#############################################################################################
-############################# Generate Output File Names ####################################
-#############################################################################################
-
-
-print(paste("folder =",folder))
-print(paste("MODELfn =",MODELfn))
-## MODELfn
-if(is.null(MODELfn)){
-	MODELfn<- paste(model.type,"_",response.type,"_",response.name,sep="")}
-
-if(identical(basename(MODELfn),MODELfn)){
-	MODELfn<-file.path(folder,MODELfn)}
-
-
-print(paste("MODELpredfn =",MODELpredfn))
-## MODELpredfn
-if(is.null(MODELpredfn)){
-	MODELpredfn<-paste(MODELfn,"_pred",sep="")}
-
-if(identical(basename(MODELpredfn),MODELpredfn)){
-	MODELpredfn<-file.path(folder,MODELpredfn)}
-
 
 
 #############################################################################################
@@ -287,7 +276,7 @@ if (prediction.type %in% c("CV","TRAIN","OOB")){
 
 	## If test data supplied give warning
 	if (!is.null(qdata.testfn)){
-		warning("if 'prediction.type' is ",prediction.type," then 'qdata.testfn' is ignored")}
+		warning("if 'prediction.type' is ",prediction.type," then 'qdata.testfn' is ignored", immediate. = WARN.IM)}
 
 	## If training data is NULL, then the user selects file from pop-up browser.
 	if (is.null(qdata.trainfn)){
@@ -319,9 +308,9 @@ if (prediction.type=="TEST"){
 
 	## If training data supplied give warning
 	if (!is.null(qdata.trainfn)){
-		warning("if 'prediction.type' is 'TEST' then 'qdata.trainfn' is ignored")}
+		warning("if 'prediction.type' is 'TEST' then 'qdata.trainfn' is ignored", immediate. = WARN.IM)}
 
-	## If training data is NULL, then the user selects file from pop-up browser.
+	## If test data is NULL, then the user selects file from pop-up browser.
 	if (is.null(qdata.testfn)){
 		if(.Platform$OS.type=="windows"){
 			qdata.testfn <- choose.files(caption="Select data file", filters = Filters["csv",], multi = FALSE)
@@ -344,6 +333,12 @@ if (prediction.type=="TEST"){
 
 }
 
+#############################################################################################
+######################### Extract response.name from model.obj ##############################
+#############################################################################################
+
+response.name<-check.response.name(model.obj=model.obj,model.type=model.type,response.name=response.name,qdata=qdata)
+
 #############################################################################################
 ###################### Check if response or any predList not in qdata #######################
 #############################################################################################
@@ -370,6 +365,29 @@ if(response.type=="categorical"){
 
 }
 
+#############################################################################################
+############################# Generate Output File Names ####################################
+#############################################################################################
+
+
+#print(paste("folder =",folder))
+#print(paste("MODELfn =",MODELfn))
+## MODELfn
+if(is.null(MODELfn)){
+	MODELfn<- paste(model.type,"_",response.type,"_",response.name,sep="")}
+
+if(identical(basename(MODELfn),MODELfn)){
+	MODELfn<-file.path(folder,MODELfn)}
+
+
+#print(paste("MODELpredfn =",MODELpredfn))
+## MODELpredfn
+if(is.null(MODELpredfn)){
+	MODELpredfn<-paste(MODELfn,"_pred",sep="")}
+
+if(identical(basename(MODELpredfn),MODELpredfn)){
+	MODELpredfn<-file.path(folder,MODELpredfn)}
+
 
 #############################################################################################
 ######################## Select unique row identifier #######################################
@@ -380,8 +398,8 @@ if (is.null(unique.rowname)){
 	if(unique.rowname!="row_index" && unique.rowname!=""){
 
 		#if(class(RF$na.action)=="omit")
-		print(paste("A: rownames length =",length(rownames(qdata)) ))
-		print(paste("A: qdata length =",length(qdata[,unique.rowname]) ))
+		#print(paste("A: rownames length =",length(rownames(qdata)) ))
+		#print(paste("A: qdata length =",length(qdata[,unique.rowname]) ))
 
 		if(anyDuplicated(qdata[,unique.rowname])){
 			stop("'unique.rowname' contains duplicated values")}
@@ -390,13 +408,13 @@ if (is.null(unique.rowname)){
 	}	
 }else{
 	if(!(unique.rowname%in%names(qdata))){
-		warning("unique.rowname ",unique.rowname," not found in qdata therefore row index numbers will be used instead")
+		warning("unique.rowname ",unique.rowname," not found in qdata therefore row index numbers will be used instead", immediate. = WARN.IM)
 		unique.rowname<-FALSE
 		rownames(qdata)<-1:nrow(qdata)
 	}
 	if(unique.rowname!=FALSE){
-		print(paste("B: rownames length =",length(rownames(qdata))))
-		print(paste("B: qdata length =",length(qdata[,unique.rowname])))
+		#print(paste("B: rownames length =",length(rownames(qdata))))
+		#print(paste("B: qdata length =",length(qdata[,unique.rowname])))
 
 		if(anyDuplicated(qdata[,unique.rowname])){
 			stop("'unique.rowname' contains duplicated values")}
@@ -408,7 +426,7 @@ if (is.null(unique.rowname)){
 ############################ check 'diagnostic.flag' ########################################
 #############################################################################################
 
-print("checking diagnostic.flag")
+#print("checking diagnostic.flag")
 
 DFLAG<-FALSE
 if(!is.null(diagnostic.flag)){DFLAG<-TRUE}
@@ -454,16 +472,16 @@ if(DFLAG){
 ################################## Drop unused columns ######################################
 #############################################################################################
 
-print("dropping unused collumns")
+#print("dropping unused columns")
 
 
-print("  qdata dimensions before")
-print(dim(qdata))
+#print("  qdata dimensions before")
+#print(dim(qdata))
 
 qdata<-qdata[,datakeep]
 
-print("  qdata dimensions after")
-print(dim(qdata))
+#print("  qdata dimensions after")
+#print(dim(qdata))
 
 
 
@@ -471,19 +489,20 @@ print(dim(qdata))
 ########## Check for factored predictors with levels not found in training data #############
 #############################################################################################
 	
-print("checking factors")
+#print("checking factors")
 
-if(!is.null(model.obj$levels)){
-	invalid.levels<-model.obj$levels
+if(!is.null(model.levels)){
+	invalid.levels<-model.levels
 
-	for(p in names(model.obj$levels)){
-		valid.levels<-c(model.obj$levels[[p]],NA)
+	for(p in names(model.levels)){
+		valid.levels<-c(model.levels[[p]],NA)
 		invalid.levels[[p]]<-unique(qdata[,p][!qdata[,p]%in%valid.levels] )
-		qdata[,p]<-factor(qdata[,p],levels=model.obj$levels[[p]])
+		qdata[,p]<-factor(qdata[,p],levels=model.levels[[p]])
 		if(length(invalid.levels[[p]])>0){
 			invalid.lev<-paste(invalid.levels[[p]],collapse=", ")
 			warning(	"categorical factored predictor ",p," contains levels: ",invalid.lev,
-					" not found in training data and these categories will be treated as NA and either omited or replaced")
+					" not found in training data and these categories will be treated as NA and either omited or replaced",
+					immediate. = WARN.IM)
 		}
 	}
 }
@@ -495,7 +514,7 @@ if(!is.null(model.obj$levels)){
 NA.resp<-is.na(qdata[,response.name])
 
 if(any(NA.resp)){
-	warning("Omiting ", sum(NA.resp), " datapoints with NA values for response")
+	warning("Omiting ", sum(NA.resp), " datapoints with NA values for response", immediate. = WARN.IM)
 	qdata <- qdata[!NA.resp,]
 }
 
@@ -503,7 +522,7 @@ if(any(NA.resp)){
 ############################### Determine NA.ACTION #########################################
 #############################################################################################
 
-print("Na action")
+#print("Na action")
 
 ###create logical vector of datarows containing NA in predictors or reponses###
 
@@ -522,37 +541,66 @@ NA.pred<-apply(qdata[,predList],1,function(x){any(is.na(x))})
 #model.NA.ACTION and model.na.action are similar but extracted from model object,
 #   and need to be checked against 'model.diagnostics(na.action)'.
 
-model.NA.ACTION<-NULL
-model.na.action<-NULL
-NA.ACTION<-NULL
+model.NA.ACTION<-NA
+model.na.action<-NA
+NA.ACTION<-NA
 
-if(any(NA.pred) || (any(var.factors) && prediction.type=="CV")){
+if(any(NA.pred) || (!is.null(model.levels) && prediction.type=="CV")){
 
-	
 	###extract na.action from model.obj and check if valid, and if argument is NULL, default to option from model.obj###
-	if(!is.null(model.obj$na.action)){
-		model.NA.ACTION<-class(model.obj$na.action)
-		print(paste("model.NA.ACTION =",model.NA.ACTION))
-		if(!model.NA.ACTION%in%c("omit","roughfix")){
-			warning("Model Object was built with 'na.action' ",model.NA.ACTION," which is not supported by model map")
-			model.NA.ACTION<-NULL
+	if(model.type!="CF"){
+		if("QRF"%in%names(model.obj)){
+			model.NA.ACTION.QRF<-NA
+			model.NA.ACTION.RF<-NA
+			if(!is.null(model.obj$QRF$na.action)){model.NA.ACTION.QRF<-class(model.obj$QRF$na.action)}
+			if(!is.null(model.obj$RF$na.action)){model.NA.ACTION.RF<-class(model.obj$RF$na.action)}
+			if(model.NA.ACTION.QRF%in%"omit" || model.NA.ACTION.RF%in%"omit"){
+				model.NA.ACTION<-"omit"
+			}else{
+				if(model.NA.ACTION.QRF%in%"roughfix" || model.NA.ACTION.RF%in%"roughfix"){model.NA.ACTION=="roughfix"}
+			}
+			if(!is.na(model.NA.ACTION) && !identical(model.NA.ACTION.QRF,model.NA.ACTION.RF)){
+				warning("QRF na.action does not match RF na.action, diagnostics will use na.action=",model.NA.ACTION)}	
 		}else{
-			model.na.action<-switch(model.NA.ACTION,omit=na.omit,roughfix=na.roughfix)
-			if(is.null(na.action)){			#if model.obj has na.action and function call does not then use action from model.obj
-				NA.ACTION<-model.NA.ACTION	#if this step is reached then whole next section not needed
-				na.action<-model.na.action
+			if(!is.null(model.obj$na.action)){model.NA.ACTION<-class(model.obj$na.action)}
+		}
+			
+		if(!is.na(model.NA.ACTION)){
+			if(!model.NA.ACTION%in%c("omit","roughfix")){
+				warning("Model Object was built with 'na.action' ",model.NA.ACTION," which is not supported by model map", immediate. = WARN.IM)
+				model.NA.ACTION<-NULL
+			}else{
+				model.na.action<-switch(model.NA.ACTION,omit=na.omit,roughfix=na.roughfix)
+				if(is.null(na.action)){			#if model.obj has na.action and function call does not then use action from model.obj
+					NA.ACTION<-model.NA.ACTION	#if this step is reached then whole next section not needed
+					na.action<-model.na.action
+				}
 			}
 		}
 	}
 
-
+	if(model.type=="CF"){
+		if(prediction.type=="OOB"){
+			if(nrow(model.obj@data@get("input"))<nrow(qdata)){
+				if(all.equal(rownames(qdata)[!rownames(qdata)%in%rownames(model.obj@data@get("input"))],names(NA.pred[NA.pred]))){
+					model.NA.ACTION<-"omit"
+					model.na.action<-na.omit
+				}else{stop("OOB predictions, but 'qdata.trainfn' does not match data used to build model")}
+			}else{
+				if(any(NA.pred)){
+					model.NA.ACTION<-"roughfix"
+					model.na.action<-na.roughfix
+				}
+			}
+		}
+	}
 
 	###Check if na.action from argument is valid###
 
 	NAvalid<-c("na.omit","na.roughfix")
 	NAwarn<-"ModelMap currently supports only \"na.omit\" and \"na.roughfix\" for 'na.action'"
 
-	if(is.null(NA.ACTION)){	
+	if(is.na(NA.ACTION)){	
 		if(is.null(na.action)){
 			na.action <- select.list(c("na.omit","na.roughfix"), title="Select na.action")
 			if(na.action=="" || is.null(na.action)){
@@ -572,15 +620,14 @@ if(any(NA.pred) || (any(var.factors) && prediction.type=="CV")){
 	}
 
 	###Check for impossible combo of model na.action, diagnostics na.action, and prediction.type=OOB
-	if(!is.null(model.NA.ACTION)){
+	if(!is.na(model.NA.ACTION)){
 		if(model.NA.ACTION=="omit" && NA.ACTION=="roughfix" && prediction.type=="OOB"){
-			warning("'model.obj' was created with 'na.action=\"na.omit\"' therefore OOB predictions not available on data rows containing NA and 'na.action' defaulting to \"na.omit\"")
+			warning("'model.obj' was created with 'na.action=\"na.omit\"' therefore OOB predictions not available on data rows containing NA and 'na.action' defaulting to \"na.omit\"", immediate. = WARN.IM)
 			NA.ACTION<-"omit"
 			na.action<-na.omit
 		}
 		
-	}
-	
+	}	
 }
 
 				#print("NA.ACTION:")
@@ -595,15 +642,15 @@ if(any(NA.pred) || (any(var.factors) && prediction.type=="CV")){
 ################################ Deal with NA's #############################################
 #############################################################################################
 
-print("starting dealing with NA")
+#print("starting dealing with NA")
 
 if(any(NA.pred)){
 
 	###na.omit###
 
 	if(NA.ACTION=="omit"){
-		print("Omitting data points with NA predictors")
-		warning("Omitting ", sum(NA.pred), " datapoints with NA values for predictors")
+		#print("Omitting data points with NA predictors")
+		warning("Omitting ", sum(NA.pred), " datapoints with NA values for predictors", immediate. = WARN.IM)
 		qdata<-na.action(qdata)
 	}
 	
@@ -611,7 +658,7 @@ if(any(NA.pred)){
 	###na.roughfix###
 
 	if(NA.ACTION=="roughfix"){
-		warning("Rough fixing ", sum(NA.pred), " data points with NA values for predictors by replacing NA with median or most common value")
+		warning("Rough fixing ", sum(NA.pred), " data points with NA values for predictors by replacing NA with median or most common value", immediate. = WARN.IM)
 
 		qdata<-na.action(qdata)
 
@@ -622,7 +669,7 @@ if(any(NA.pred)){
 	}
 }
 
-print("done dealing with NA")
+#print("done dealing with NA")
 
 
 ##for OOB models, check number of rows is equal to data used in building model
@@ -632,43 +679,50 @@ print("done dealing with NA")
 #	}
 #}
 
+#############################################################################################
+#### If predictions are OOB/CV/TRAIN check if qdata.train and model have same responses #########
+#############################################################################################
+
+#nice idea, but messes up when model and diagnostics have different na.action
+
+#if(prediction.type%in%c("OOB","CV","TRAIN")){
+#	respdat<-NULL
+#	if(model.type!="CF"){
+#		if(!is.null(model.obj$predictor.data)){
+#			respdat<-model.obj$predictor.data[,response.name]
+#		}else{
+#			if(model.type=="RF"){ respdat<-model.obj$y}
+#			if(model.type=="QRF"){respdat<-model.obj$y}
+#			if(model.type=="SGB"){
+#				respdat<-model.obj$data$y
+#				if(is.null(respdat)){
+#					warning("SGB model built with 'keep.data=FALSE' therefore can not check if supplied qdata.trainfn matches data used to build original model",immediate. = TRUE)
+#				}
+#			}
+#		}
+#	}else{
+#		respdat<-model.obj@data@get("response")[[1]] #note-may need to adjust for multivariate responses
+#	}
+#	if(!identical(respdat,qdata[,response.name])){
+#		stop("supplied qdata.trainfn has different response values than data used to build model")
+#	}
+#}
+
+
 
 #############################################################################################
 ############################# Check Device Type #############################################
 #############################################################################################
 
-print("check device type")
-
-if(is.null(device.type)){
-	device.type <- select.list(c("default","jpeg","none","pdf","postscript"), title="Diagnostic Output?", multiple = TRUE)
-	device.type <- c(device.type,"default")
-}
-if(length(device.type)==0 || is.null(device.type)){
-	device.type <- "default"
-}
-
-if(!is.null(device.type)){
-	device.type[device.type=="windows"]<-"default"
-	if(any(!device.type%in%c("default","jpeg","none","pdf","postscript"))){
-		stop("illegal 'device.type' device types must be one or more of 'default' 'jpeg' 'pdf' or 'postscript'")
-	}
-	device.type<-sort(device.type)
-	if("default"%in%device.type){
-		device.type<-c(device.type[device.type!="default"],"default")
-	}
-}
-
-if("none"%in%device.type){
-	device.type<-"none"
-}
-
+device.type<-check.device.type(device.type)
 
+if(is.null(res)){res<-jpeg.res}
 
 #############################################################################################
 ############################# SGB + CV: check for n.trees ###################################
 #############################################################################################
 
-print("check n.trees")
+#print("check n.trees")
 
 if(model.type=="SGB"){
 	if(is.null(n.trees)){
@@ -690,12 +744,13 @@ if(model.type=="SGB"){
 ############################ Make validation predictions ####################################
 #############################################################################################
 
-
 if(!is.null(seed)){
 	set.seed(seed)}
 
+#print("starting predictions")
 
-print("starting predictions")
+LOWER<-0.10
+UPPER<-0.90
 
 PRED <- prediction.model(		model.obj=model.obj,
 						model.type=model.type,
@@ -715,13 +770,48 @@ PRED <- prediction.model(		model.obj=model.obj,
 						model.na.action=model.na.action,			
 						model.NA.ACTION=model.NA.ACTION,
 
+						model.levels=model.levels,
+
+					# QRF arguments
+						quantiles = quantiles,
+						all=all,
+						LOWER=LOWER,
+						UPPER=UPPER,
+
+					# CF arguments:
+						subset=NULL, 
+						weights=NULL,
+						controls=controls,
+						xtrafo=xtrafo, 
+						ytrafo=ytrafo,
+						scores=NULL,
+
 					# SGB arguments
 						n.trees=n.trees
 						)
 
-
 ###if na.action="na.roughfix" must turn estimated responses back to NA
 
+#############################################################################################
+######################### Correlation plot ##################################################
+#############################################################################################
+
+if(DFLAG){qdata.corr<-qdata[,diagnostic.flag]}else{qdata.corr<-qdata}
+
+correlation.function(	qdata=qdata.corr,
+				predList=predList,
+				predFactor=predFactor,
+				MODELpredfn=MODELpredfn,
+
+				device.type=device.type,
+				res=res,
+				device.width=device.width,
+				device.height=device.height,
+				units=units,
+				pointsize=pointsize,
+				cex=cex	
+				)
+
 
 #############################################################################################
 ########################### If DFLAG=T remove unused rows ###################################
@@ -734,67 +824,59 @@ if(DFLAG){
 	write.table(PRED.OUT,file=PREDICTIONfn,sep=",",row.names=FALSE)
 }
 
-
-
-
-
-print("removing rows")
-
-#print("PRED dimensions before:")
-#print(dim(PRED))
-
 if(DFLAG){
 	PRED<-PRED[qdata[,diagnostic.flag],]
 }
 
 
-#print("PRED dimensions after:")
-#print(dim(PRED))
-#print(PRED)
-
-#print(paste("diagnostic.flag =",diagnostic.flag))
-
-#print("names(qdata)=")
-#print(names(qdata))
-
-#print("qdata[,diagnostic.flag]")
-#print(qdata[,diagnostic.flag])
 #############################################################################################
 ################################### Run Diagnostics #########################################
 #############################################################################################
 
 
 
-print("starting diagnostics")
+#print("starting diagnostics")
 
 diagnostics.function(	model.obj=model.obj,
 				model.type=model.type,
 				predList=predList,
+				predFactor=predFactor,
 				PRED=PRED,
+				qdata=qdata.corr,
 				MODELfn=MODELfn,
 				MODELpredfn=MODELpredfn,
 				response.name=response.name,
 				response.type=response.type,
 				prediction.type=prediction.type,
+				quantiles=quantiles,
+				imp.quantiles=imp.quantiles,
+				LOWER=LOWER,
+				UPPER=UPPER,
 				folder=folder,
+
+			# Graphics Arguments
 				device.type=device.type,
-				jpeg.res=jpeg.res,
+				res=res,
 				device.width=device.width,
 				device.height=device.height,
+				units=units,
+				pointsize=pointsize,
 				cex=cex,
+
 				req.sens=req.sens,
 				req.spec=req.spec,
 				FPC=FPC,
 				FNC=FNC)
 #if(.Platform$OS.type=="windows"){bringToTop(which = dev.cur(), stay = FALSE)}
 
-print("ending diagnostics")
+#print("ending diagnostics")
 
 #############################################################################################
 ################################## Write a list of argumets #################################
 #############################################################################################
 
-print("starting argument record")
+
+#print("starting argument record")
 ARGfn<-paste(MODELfn,"_model_diagnostics_arguments.txt",sep="")
 
 A<-formals(model.diagnostics)
@@ -817,12 +899,23 @@ A<-A[c(length(A),1:(length(A)-1))]
 
 #print(A)
 #capture.output(print(A),file=ARGfn)
-print("ending argument record")
+#print("ending argument record")
+
+#############################################################################################
+##################################### Warnings ##############################################
+#############################################################################################
+
+#if(!is.null(WARN)){
+#	for(w in 1:length(WARN)){
+#		warning(WARN[w])
+#	}
+#}
 
 #############################################################################################
 ##################################### Returns ###############################################
 #############################################################################################
 
+#PRED<-PRED[,!names(PRED)%in%c("lower","median","upper")]
 
 return(PRED)
 }
